[−][src]Crate egg
EGraphs (and almost everything else in this crate) are
parameterized over the language given by the user (by implementing
the Language trait).
A typically usage would either implement Language or use the
provided TestLang. From there, you can use the functionality
from the ParsableLanguage trait module to create expressions
and add them to the EGraph.
Modules
| dot | EGraph visualization with GraphViz |
| egraph | |
| expr | |
| extract | |
| parse | |
| pattern | |
| util |